@@ -223,6 +223,11 @@ loop(int sd_bind)
 						session.client,
 						sizeof(session.client));
 
+				if ((topt & T_DROP_UNKNOWN_USER) && !rs) {
+				  service_unavailable(sd_client);
+				  quit(0);
+				}
+
 				parse_transmap(&cs,
 					       rs ? session.client : 0,
 					       inet_ntoa(addr.sin_addr),
@@ -230,7 +235,6 @@ loop(int sd_bind)
 					       sizeof(session.client));
 				
 				if (cs == 1) {
-					topt |= T_SUPERCLIENT;
 					anubis_changeowner(session.client);
 					auth_tunnel();
 				}
@@ -239,7 +243,6 @@ loop(int sd_bind)
 					 && ntohl(addr.sin_addr.s_addr)
 					 == INADDR_LOOPBACK) {
 					if (check_username(session.client)) {
-						topt |= T_SUPERCLIENT;
 						anubis_changeowner(session.client);
 						auth_tunnel();
 					}
@@ -317,7 +320,7 @@ loop(int sd_bind)
 					remote_client = (void *)sd_client;
 					remote_server = (void *)sd_server;
 					alarm(900);
-					smtp_session((void *)sd_client, (void *)sd_server);
+					smtp_session();
 					alarm(0);
 
 #ifdef USE_SSL
@@ -388,7 +391,7 @@ _stdio_read(void *sd, char *data, size_t size, size_t *nbytes)
 	FD_SET(fd, &rds);
 	do
 		n = select(fd + 1, &rds, NULL, NULL, NULL);
-	while (errno == EINTR);
+	while (n < 0 && errno == EINTR);
 	if (n > 0) {
 		n = read(fd, data, size);
 		if (n >= 0) 
@@ -403,30 +406,6 @@ _stdio_strerror(int rc)
 	return strerror(rc);
 }
 
-#if defined(O_NONBLOCK)
-# define FCNTL_NONBLOCK O_NONBLOCK
-#elif defined(O_NDELAY)
-# define FCNTL_NONBLOCK O_NDELAY
-#else
-# error "Neither O_NONBLOCK nor O_NDELAY are defined"
-#endif
-
-int
-set_nonblocking(int fd)
-{
-	int flags;
-	
-	if ((flags = fcntl(fd, F_GETFL, 0)) < 0) {
-		perror("F_GETFL");
-		return -1;
-	}
-	if (fcntl(fd, F_SETFL, flags | FCNTL_NONBLOCK) < 0) {
-		perror("F_GETFL");
-		return -1;
-	}
-	return 0;
-}
-
 void
 stdinout(void)
 {
@@ -456,13 +435,11 @@ stdinout(void)
 		free_mem();
 		return;
 	}
-	set_nonblocking(sd_client);
-	set_nonblocking(sd_server);
 	remote_client = (void *)sd_client;
 	remote_server = (void *)sd_server;
 	net_set_io(CLIENT, _stdio_read, _stdio_write, NULL, _stdio_strerror);
 	net_set_io(SERVER, _stdio_read, _stdio_write, NULL, _stdio_strerror);
-	smtp_session((void *)sd_client, (void *)sd_server);
+	smtp_session();
 
 	close_socket(sd_server);
 	free_mem();
