@@ -26,11 +26,9 @@
 
 #include <config.h>
 
-# if defined(HAVE_LIBGCRYPT) && defined(HAVE_LIBGNUTLS)
-#  if defined(HAVE_GNUTLS_GNUTLS_H)
-#   define HAVE_GNUTLS
-#  endif /* HAVE_GNUTLS_GNUTLS_H */
-# endif /* HAVE_LIBGCRYPT and HAVE_LIBGNUTLS */
+# if defined(HAVE_GNUTLS_GNUTLS_H)
+#  define HAVE_GNUTLS
+# endif /* HAVE_GNUTLS_GNUTLS_H */
 # if defined(HAVE_LIBSSL) && defined(HAVE_LIBCRYPTO)
 #  if defined(HAVE_OPENSSL_SSL_H)
 #   define HAVE_OPENSSL
@@ -208,8 +206,9 @@
 #define ALL   2
 
 /* error types */
-#define SOFT 0
-#define HARD 1
+#define SOFT   1
+#define HARD   2
+#define SYNTAX 3
 
 /* bit values for topt */
 #define T_ERROR             0x00000001
@@ -228,7 +227,7 @@
 #define T_LOCAL_MTA         0x00002000
 #define T_ALLOW_LOCAL_MTA   0x00004000
 #define T_TRANSLATION_MAP   0x00008000
-#define T_SUPERCLIENT       0x00010000
+#define T_DROP_UNKNOWN_USER 0x00010000
 #define T_USER_NOTPRIVIL    0x00020000
 #define T_STARTTLS          0x00040000
 #define T_ESMTP_AUTH        0x00080000
@@ -238,10 +237,20 @@
 #define T_RELAX_PERM_CHECK  0x00800000
 #define T_ENTIRE_BODY       0x01000000
 
-/* Tags */
-#define R_BASIC             0x00000001
-#define R_PERLRE            0x00000002
-#define R_SCASE             0x00000004
+/* Regexp modifiers */
+/* Basic types */
+#define R_EXACT             0x00000001
+#define R_POSIX             0x00000002
+#define R_PERLRE            0x00000004
+/* Other modifiers */
+#define R_BASIC             0x00000010
+#define R_SCASE             0x00000020
+#define R_TYPEMASK          0x0000000f
+
+#define re_set_type(m,t) ((m) = ((m) & ~R_TYPEMASK) | ((t) & R_TYPEMASK))
+#define re_typeof(m) ((m) & R_TYPEMASK)
+#define re_set_flag(m,f) ((m) |= (f))
+#define re_clear_flag(m,f) ((m) &= ~(f))
 
 /* A special header used by Anubis to implement rules. */
 #define X_ANUBIS_RULE_HEADER "\nRULE\n"
@@ -322,13 +331,14 @@ void parse_transmap(int *, char *, char *, char *, int);
 void translate_section_init(void);
 
 /* tunnel.c */
-void smtp_session(void *, void *);
+void smtp_session(void);
 
 /* message.c */
 void message_add_body(MESSAGE *, char *, char *);
 void message_add_header(MESSAGE *, char *, char *);
-void message_remove_headers(MESSAGE *, char *);
-void message_modify_headers(MESSAGE *, char *, char *, char *);
+void message_remove_headers(MESSAGE *, RC_REGEX *);
+void message_modify_headers(MESSAGE *, RC_REGEX *, char *, char *);
+void message_modify_body(MESSAGE *, RC_REGEX *, char *);
 void message_external_proc(MESSAGE *, char **);
 void message_init(MESSAGE *);
 void message_free(MESSAGE *);
@@ -356,14 +366,16 @@ void change_to_lower(char *);
 
 /* files.c */
 void message_append_text_file(MESSAGE *, char *);
-void message_append_signature_file(MESSAGE *, char *);
+void message_append_signature_file(MESSAGE *);
 
 /* regex.c */
 int anubis_regex_match(RC_REGEX *, char *, int *, char ***);
 RC_REGEX *anubis_regex_compile(char *, int);
-void anubis_regex_free(RC_REGEX *);
+void anubis_regex_free(RC_REGEX **);
 char *anubis_regex_source(RC_REGEX *);
 int anubis_regex_refcnt(RC_REGEX *);
+char *anubis_regex_replace(RC_REGEX *, char *, char *);
+void anubis_regex_print(RC_REGEX *);
 
 /* rc.c */
 void rc_system_init(void);
