@@ -104,7 +104,7 @@ enum rc_inst_opcode {        /* Operation code */
 struct rc_inst {             /* Instruction definition */
 	enum rc_inst_opcode opcode;
 	int part;            /* Message part to operate upon */ 
-	char *key;           /* Key */
+	RC_REGEX *key;       /* Key */
 	char *key2;          /* New key value (for modify) */
 	char *arg;           /* Argument */
 };
@@ -141,22 +141,37 @@ struct rc_secdef_child {
 	void *data;
 };
 
-struct rc_secdef {
-	char *name;
-	struct rc_secdef_child *child;
+/* Section priorities affect linking the user-defined sections to
+   the parse tree left from parsing the system configuration file. */
+enum section_prio {
+	prio_user_only, /* Only user-defined section is taken into account */
+	prio_system_only,/* Only system-defined section */
+	prio_system,    /* System-defined section first, user-defined next */
+	prio_user       /* User-defined section first, system-defined next */
 };
 
+struct rc_secdef {
+	char *name;                   /* Section name */
+	enum section_prio prio;       /* Execution priority */ 
+	int allow_prog;               /* Are rules allowed in this section? */
+	struct rc_secdef_child *child;
+};
+	
 /* Global data */
 extern int cfg_line_num;
 extern char *cfg_file; 
 
 /* Function declarations */
 void verbatim(void);
+void lex_clear_state(void);
+
+void error_sync_begin();
+
 RC_SECTION *rc_section_lookup(RC_SECTION *, char *);
 void rc_section_link(RC_SECTION **, RC_SECTION *);
 void rc_secdef_add_child(struct rc_secdef *, struct rc_secdef_child *);
 RC_SECTION *rc_parse(char *);
-void rc_section_list_destroy(RC_SECTION *);
+void rc_section_list_destroy(RC_SECTION **);
 int rc_run_cond(char *, int, char *);
 void rc_run_section_list(int, RC_SECTION *, struct rc_secdef *);
 void rc_run_section(int, RC_SECTION *, struct rc_secdef *, void *, MESSAGE *);
@@ -164,4 +179,6 @@ void rc_call_section(int, RC_SECTION *, struct rc_secdef *, void *, MESSAGE *);
 void rc_set_debug_level(char *);
 int rc_open(char *);
 struct rc_secdef *anubis_add_section(char *);
+struct rc_secdef *anubis_find_section(char *);
 
+void parse_error(const char *fmt, ...);
